      subroutine ellipse(xcent,ycent,a,b,phi)
c       ---- APM routine (Mike Irwin, Jon Loveday et.al.) ----
c *** ELLIPSE plots ellipse with semimajor and minor axes a,b
c ***         and angle of major axis to x axis phi (in degrees) .

c     Note: pgpoly is used => pgsfs will determine filling style
c     FS = 1 => solid (default)  FS = 2 => outline
c     FS = 3 => hatched          FS = 4 => cross-hatched
      
      real*4 x(100),y(100)
c      print*,'beg of ellipse'

      pi_180=atan(1.0)/45.0
      phi_loc=pi_180*phi
      cphi=cos(phi_loc)
      sphi=sin(phi_loc)
      xcent=xcent+0.5
      ycent=ycent+0.5

c *** step size in direction of tangent
c      nstep=min0(100,nint(3.0*amax1(sqrt(a*b),3.0)))
      nstep=min(100,nint(3.0*max(sqrt(a*b),3.0)))
      stepsz=360.0*pi_180/float(nstep-1)
      do 100 i=1,nstep
         angle=(i-1)*stepsz
         xx=a*cos(angle)
         yy=b*sin(angle)
         x(i)=xcent+xx*cphi-yy*sphi
         y(i)=ycent+xx*sphi+yy*cphi
  100 continue
c      call pgline(nstep,x,y)

      call pgpoly(nstep,x,y)
      return
      end


      subroutine ellipse_radec(xcent,ycent,a,b,phi)
c       ---- APM routine (Mike Irwin, Jon Loveday et.al.) ----
c     modified B.Roukema
c *** ELLIPSE plots ellipse with semimajor and minor axes a,b
c ***         and angle of major axis to x axis phi (in degrees) .

c     xcent is presumed to be right ascension in hours
c     ycent is presumed to be declination in degrees

c     a  is presumed to be in seconds of arc
c     output plot is in time-seconds (ra) and seconds of arc (dec)

c     Note: pgpoly is used => pgsfs will determine filling style
c     FS = 1 => solid (default)  FS = 2 => outline
c     FS = 3 => hatched          FS = 4 => cross-hatched
      
      parameter(pi=3.14159265358979)
      parameter(pi_180= pi/180.0) ! conversion deg -> rad


      real*4 x(100),y(100)

c      factor= cos(ycent*pi_180)/15.0 ! reduce "x" direction by factor
      factor= 1.0/15.0 ! reduce "x" direction by factor
      factor= -factor           ! invert for ra increasing to left

      conv=atan(1.0)/45.0
      phi_loc=conv*phi
      cphi=cos(phi_loc)
      sphi=sin(phi_loc)
c      xcent=xcent+0.5 ! only due to pixelisation, no good if not pixels
c      ycent=ycent+0.5
c *** step size in direction of tangent
      nstep=min0(100,nint(3.0*amax1(sqrt(a*b),3.0)))
      stepsz=360.0*conv/float(nstep-1)
      do 100 i=1,nstep
         angle=(i-1)*stepsz
         xx=a*cos(angle)
         yy=b*sin(angle)
         x(i)=xcent*3600.0+ (xx*cphi-yy*sphi) *factor
         y(i)=ycent*3600.0+xx*sphi+yy*cphi
  100 continue
c      call pgline(nstep,x,y)
      call pgpoly(nstep,x,y)
      return
      end

